###############################################################################
# Objective: Generic
# Constraints: Simple bounds
###############################################################################
# Pollution control problem
#    Coded by A. Ismael F. Vaz 27/02/2004  aivaz@dps.uminho.pt
#    University of Minho, Portugal
#
# Minimum stack height at ground level
#
# Adapted for multilocal optimization by A. Ismael F. Vaz 14/04/2004
#
# This is a regular (finite) maximization problem
#
###############################################################################

# Pi
param pi := 4*atan(1);
#number of pollution emission sources
param nsources;
# Wind direction
param theta;
# Mean wind speed
param U;
# Environment temperature (Kelvin)
param Te;
# Position of sources
param xpos{1..nsources};
param ypos{1..nsources};
# diameter
param diam{1..nsources};
# emission rates
param Q{1..nsources};
# Gas velocity
param V{1..nsources};
# Temperature
param Ts{1..nsources};




#number of t
param nt;


# starting point 
param h{1..nsources};

# t variable t[1]-> x coordenate
# t variable t[2]-> y coordenate
var t {i in 1..nt};

# delta H paramenters (plume), Holland equation
param dH {i in 1..nsources} := (V[i]*diam[i]/U)*
			(1.5+2.68*diam[i]*(Ts[i]-Te)/Ts[i]);


# Translation and rotation for sigma and concentration
var X{i in 1..nsources} = (t[1]-xpos[i])*cos(theta)-(t[2]-ypos[i])*sin(theta);
var Y{i in 1..nsources} = (t[1]-xpos[i])*sin(theta)+(t[2]-ypos[i])*cos(theta);

# Standard deviations
var sigmaY{i in 1..nsources} =
	if(X[i]<=10) then 0.9591
	 else   if(X[i]<=2000)   then 0.1136*X[i]^0.9265
	  else  if(X[i]<=10000)  then 0.1385*X[i]^0.9015
	   else if(X[i]<=100000) then 0.2030*X[i]^0.8600
	    else 1; /* avoid division by zero */

var sigmaZ{i in 1..nsources} =
	(if(X[i]<=10) then 0.07925
	 else    if(X[i]<=200)    then 4.828E-5*(log(X[i]))^8.8766
	  else   if(X[i]<=1000)   then 3.108E-6*(log(X[i]))^10.5295
	   else  if(X[i]<=5000)   then 1.808E-7*(log(X[i]))^11.9998
	    else if(X[i]<=100000) then 1.892E-9*(log(X[i]))^14.1284
	     else 0)/sigmaY[i];

var sigmaZY{i in 1..nsources} =
	(if(X[i]<=10) then 0.07925
	 else    if(X[i]<=200)    then 4.828E-5*(log(X[i]))^8.8766
	  else   if(X[i]<=1000)   then 3.108E-6*(log(X[i]))^10.5295
	   else  if(X[i]<=5000)   then 1.808E-7*(log(X[i]))^11.9998
	    else if(X[i]<=100000) then 1.892E-9*(log(X[i]))^14.1284
	     else 0);

# Concentration of gas
var C {i in 1..nsources} = (0.8*Q[i]/(pi*sigmaZY[i]*U))*
	exp(-0.5*((Y[i]/sigmaY[i])^2+((h[i]+dH[i])/sigmaZ[i])^2));



maximize ft:
	sum {i in 1..nsources} (C[i]);

subject to tbound1:
	-20000 <= t[1] <= 20000;
subject to tbound2:
	-20000 <= t[2] <= 20000;

data;

param nsources=10;
param nt := 2;

param Te    := 283;     # Environment temperature
param theta := 3.996;   # Wind direction in rad
param U     := 5.64;    # Mean wind speed


param xpos :=		# x position on the ground
	1	-3000
	2	-2600
	3	-1100
	4	 1000
	5	 1000
	6	 2700
	7	 3000
	8	-2000
	9	 0
	10	 1500;

param ypos :=		# y position on the ground
	1	-2500
	2	-300
	3	-1700
	4	-2500
	5	 2200
	6	 1000
	7	-1600
	8	 2500
	9	 0
	10	-1600;

param h :=		# Pollution source height
	1	196.93
	2	380.06
	3	403.12
	4	428.38
	5	344.81
	6	274.58
	7	402.83
	8	396.82
	9	415.58
	10	423.99;

param Ts :=		# Temperature of gas in source
	1	413
	2	413
	3	413
	4	413
	5	413
	6	413
	7	413
	8	413
	9	413
	10	413;

param diam :=		# Pollution source diameter
	1	8
	2	8
	3	7.6
	4	7.6
	5	6.3
	6	6.3
	7	4.3
	8	4.3
	9	5
	10	5;

param Q :=		# Uniform emission rate
	1	2882.6
	2	2882.6
	3	2391.3
	4	2391.3
	5	2173.9
	6	2173.9
	7	1173.9
	8	1173.9
	9	1304.3
	10	1304.3;

param V :=
	1	19.245
	2	19.245
	3	17.690
	4	17.690
	5	23.404
	6	23.404
	7	27.128
	8	27.128
	9	22.293
	10	22.293;

option solver mlocpsoa;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display t;
display ft;
